#pragma once

#include "../../core/Time.h"

#include "../Commit.h"
#include "detail/MemoryItem.h"


namespace armarx::armem::base
{
    /**
     * @brief Data of a single entity instance.
     */
    template <class _DerivedT>
    class EntityInstanceBase :
        virtual public detail::MemoryItem
    {
        using Base = detail::MemoryItem;

    public:

        using DerivedT = _DerivedT;


    public:

        EntityInstanceBase& operator=(const EntityInstanceBase& other)
        {
            //other._copySelf(*this);
            return *this;
        }


        inline int& index()
        {
            return id().instanceIndex;
        }
        inline int index() const
        {
            return id().instanceIndex;
        }

        // Copying
        virtual DerivedT copy() const
        {
            DerivedT d;
            this->_copySelf(d);
            return d;
        }

        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        virtual void update(const EntityUpdate& update, int index) = 0;


        virtual bool equalsDeep(const DerivedT& other) const = 0;


        std::string getLevelName() const override
        {
            return "entity instance";
        }

        std::string getKeyString() const override
        {
            return std::to_string(index());
        }


    protected:

        virtual void _copySelf(DerivedT& other) const
        {
            Base::_copySelf(other);
        }

    };

}
