#pragma once

#include <RobotAPI/libraries/armem/core/error/ArMemError.h>

#include "MemoryItem.h"


namespace armarx::armem::base::detail
{

    /**
     * @class Provides default implmentations of `MemoryContainer`, as well as
     * iterators (which requires a template).
     */
    template <class _ContainerT, class _DerivedT>
    class MemoryContainerBase :
        virtual public MemoryItem
    {
        using Base = MemoryItem;

    public:

        using DerivedT = _DerivedT;
        using ContainerT = _ContainerT;


    public:

        MemoryContainerBase()
        {}
        MemoryContainerBase(const MemoryContainerBase<ContainerT, DerivedT>& o) :
            MemoryItem(o.id),
            _container(o._container)
        {}


        MemoryContainerBase& operator=(const MemoryContainerBase& other)
        {
            other._copySelf(*this);
            return *this;
        }


        // Container methods
        virtual bool empty() const
        {
            return _container.empty();
        }
        virtual std::size_t size() const
        {
            return _container.size();
        }
        virtual void clear()
        {
            return _container.clear();
        }

        typename ContainerT::const_iterator begin() const
        {
            return _container.begin();
        }
        typename ContainerT::iterator begin()
        {
            return _container.begin();
        }
        typename ContainerT::const_iterator end() const
        {
            return _container.end();
        }
        typename ContainerT::iterator end()
        {
            return _container.end();
        }

        const ContainerT& container() const
        {
            return _container;
        }
        ContainerT& container()
        {
            return _container;
        }


        // Copying
        virtual DerivedT copy() const
        {
            DerivedT d;
            this->_copySelf(d);
            return d;
        }

        /// Make a copy not containing any elements.
        virtual DerivedT copyEmpty() const
        {
            DerivedT d;
            this->_copySelfEmpty(d);
            return d;
        }


    protected:

        /**
         * @throw `armem::error::ContainerNameMismatch` if `gottenName` does not match `actualName`.
         */
        void _checkContainerName(const std::string& gottenName, const std::string& actualName,
                                 bool emptyOk = true) const
        {
            if (!((emptyOk && gottenName.empty()) || gottenName == actualName))
            {
                throw armem::error::ContainerNameMismatch(gottenName, this->getLevelName(), actualName);
            }
        }

        virtual void _copySelf(DerivedT& other) const
        {
            Base::_copySelf(other);
            other.container() = _container;
        }

        virtual void _copySelfEmpty(DerivedT& other) const
        {
            Base::_copySelf(other);
        }


    protected:

        ContainerT _container;

    };

}
