#pragma once

#include "EntityContainerBase.h"

#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>

namespace armarx::armem::base::detail
{

    /**
     * @brief An entity container with a specific (Aron) type.
     */
    template <class _ValueT, class _EntityT, class _Derived>
    class TypedEntityContainerBase :
        virtual public EntityContainerBase<_ValueT, _EntityT, _Derived>
    {
        using Base = EntityContainerBase<_ValueT, _EntityT, _Derived>;


    public:

        using typename Base::DerivedT;
        using typename Base::ValueT;
        using typename Base::EntityT;
        using typename Base::EntitySnapshotT;
        using typename Base::EntityInstanceT;


        TypedEntityContainerBase(aron::typenavigator::ObjectNavigatorPtr aronType = nullptr) :
            _aronType(aronType)
        {}

        TypedEntityContainerBase(const TypedEntityContainerBase<_ValueT, _EntityT, DerivedT>& other) :
            MemoryItem(other),
            MemoryContainerBase<std::map<std::string, _ValueT>, DerivedT>(other),
            _aronType(other._aronType)
        {}

        TypedEntityContainerBase(const MemoryID& id, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr) :
            MemoryItem(id),
            _aronType(aronType)
        {}


        TypedEntityContainerBase& operator=(const TypedEntityContainerBase& other)
        {
            other._copySelf(*this);
            return *this;
        }


        bool hasAronType() const
        {
            return _aronType != nullptr;
        }
        aron::typenavigator::ObjectNavigatorPtr& aronType()
        {
            return _aronType;
        }
        aron::typenavigator::ObjectNavigatorPtr aronType() const
        {
            return _aronType;
        }

        virtual DerivedT copyEmpty() const override
        {
            DerivedT d;
            this->_copySelfEmpty(d);
            return d;
        }


    protected:

        virtual void _copySelf(DerivedT& other) const override
        {
            Base::_copySelf(other);
            other._aronType = _aronType;
        }

        virtual void _copySelfEmpty(DerivedT& other) const override
        {
            Base::_copySelfEmpty(other);
            other._aronType = _aronType;
        }


    protected:

        /// The expected Aron type. May be nullptr, in which case no type information is available.
        aron::typenavigator::ObjectNavigatorPtr _aronType;

    };


}
