#pragma once

#include <filesystem>

#include "../base/CoreSegmentBase.h"
#include "detail/TypedEntityContainer.h"

#include "ProviderSegment.h"

#include "../workingmemory/CoreSegment.h"


namespace armarx::armem::d_ltm
{

    /**
     * @brief Data of a core segment containing multiple provider segments.
     */
    class CoreSegment :
        virtual public base::CoreSegmentBase<ProviderSegment, CoreSegment>,
        virtual public detail::TypedEntityContainer<ProviderSegment, CoreSegment>
    {
        using Base = base::CoreSegmentBase<ProviderSegment, CoreSegment>;

    public:
        CoreSegment();
        CoreSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        CoreSegment(const std::string& name, const MemoryID& parentID, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        CoreSegment(const MemoryID& id, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);

        CoreSegment(const CoreSegment& other);
        CoreSegment& operator=(const CoreSegment& other);

        // Conversion
        wm::CoreSegment convert() const;

        // Filesystem connection
        void reload(const std::shared_ptr<std::filesystem::path>&);
        void append(const wm::CoreSegment&);

    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;
    };

}
