#include "Entity.h"

namespace armarx::armem::d_ltm
{

    Entity::Entity()
    {
    }

    Entity::Entity(const std::string& name, const MemoryID& parentID) :
        base::detail::MemoryItem(parentID.withEntityName(name))
    {
    }

    Entity::Entity(const MemoryID& id) :
        base::detail::MemoryItem(id)
    {
    }

    Entity::Entity(const Entity& other) :
        base::detail::MemoryItem(other),
        base::detail::MemoryContainerBase<std::map<Time, EntitySnapshot>, Entity>(other)
    {
    }

    Entity& Entity::operator=(const Entity& other)
    {
        other._copySelf(*this);
        return *this;
    }

    std::filesystem::path Entity::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path Entity::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName / id().coreSegmentName / id().providerSegmentName / id().entityName;
    }

    wm::Entity Entity::convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const
    {
        wm::Entity m;
        for (const auto& [_, s] : _container)
        {
            m.addSnapshot(s.convert(expectedStructure));
        }
        return m;
    }

    void Entity::reload(const std::shared_ptr<std::filesystem::path>& p_ptr)
    {
        if (!p_ptr)
        {
            ARMARX_WARNING << "The entered is NULL.";
        }
        std::filesystem::path p = _fullPath(*p_ptr);
        if (std::filesystem::is_regular_file(p))
        {
            ARMARX_ERROR << "The entered path is leading to a file! Abort due to error.";
        }

        _container.clear();
        path = p_ptr;

        if (!std::filesystem::exists(p))
        {
            ARMARX_INFO << "The entered path does not exist. Assuming an empty container.";
        }
        else
        {
            for (const auto& d : std::filesystem::directory_iterator(p))
            {
                if (d.is_directory())
                {
                    std::string k = d.path().filename();
                    armem::Time t = armem::Time::microSeconds(std::stol(k));
                    auto wms = _container.emplace(std::make_pair(t, id().withTimestamp(t)));
                    wms.first->second.reload(p_ptr);
                }
            }
        }
    }

    void Entity::append(const wm::Entity& m)
    {
        std::filesystem::create_directories(_fullPath());
        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.setTo(s);
            }
            else
            {
                std::filesystem::create_directory(_fullPath() / std::to_string(k.toMicroSeconds()));
                auto wms = _container.emplace(std::make_pair(k, id().withTimestamp(k)));
                wms.first->second.path = path;
                wms.first->second.setTo(s);
            }
        }
    }
}
