#pragma once

#include <filesystem>

#include "../base/EntityInstanceBase.h"

#include "../workingmemory/EntityInstance.h"

namespace armarx::armem::d_ltm
{
    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance :
        virtual public base::EntityInstanceBase<EntityInstance>
    {
        using Base = base::EntityInstanceBase<EntityInstance>;

    public:
        EntityInstance();
        EntityInstance(const EntityInstance&);
        EntityInstance(int index, const MemoryID& parentID = {});
        EntityInstance(const MemoryID& id);

        EntityInstance& operator=(const EntityInstance& other);

        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        virtual void update(const EntityUpdate& update, int index) override;

        virtual bool equalsDeep(const EntityInstance& other) const override;

        virtual EntityInstance copy() const override;

        // Conversion
        wm::EntityInstance convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const;

        // Filesystem connection
        void reload(const std::shared_ptr<std::filesystem::path>&);
        void setTo(const wm::EntityInstance&);

    protected:
        virtual void _copySelf(EntityInstance& other) const override;

    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

        wm::EntityInstance unwrapData(const aron::datanavigator::DictNavigatorPtr&) const;
        aron::datanavigator::DictNavigatorPtr wrapData(const wm::EntityInstance&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;

    private:
        static const constexpr char* DATA_FILENAME = "data";
        static constexpr const char* DATA_WRAPPER_DATA_FIELD            = "__ARON_DATA";
        static constexpr const char* DATA_WRAPPER_TIME_STORED_FIELD     = "__WRITER_METADATA__TIME_STORED";
        static constexpr const char* DATA_WRAPPER_TIME_CREATED_FIELD    = "__ENTITY_METADATA__TIME_CREATED";
        static constexpr const char* DATA_WRAPPER_TIME_SENT_FIELD       = "__ENTITY_METADATA__TIME_SENT";
        static constexpr const char* DATA_WRAPPER_TIME_ARRIVED_FIELD    = "__ENTITY_METADATA__TIME_ARRIVED";
        static constexpr const char* DATA_WRAPPER_CONFIDENCE_FIELD      = "__ENTITY_METADATA__CONFIDENCE";
    };
}
