#pragma once

#include <filesystem>

#include "../base/EntitySnapshotBase.h"

#include "EntityInstance.h"

#include "../workingmemory/EntitySnapshot.h"


namespace armarx::armem::d_ltm
{

    /**
     * @brief Data of an entity at one point in time.
     */
    class EntitySnapshot :
        virtual public base::EntitySnapshotBase<EntityInstance, EntitySnapshot>
    {
        using Base = base::EntitySnapshotBase<EntityInstance, EntitySnapshot>;

    public:
        EntitySnapshot();
        EntitySnapshot(Time time, const MemoryID& parentID = {});
        EntitySnapshot(const MemoryID& id);

        /// Copy the instances from `other` to this.
        EntitySnapshot(const EntitySnapshot& other);

        /// Copy the instances from `other` to this.
        EntitySnapshot& operator=(const EntitySnapshot& other);

        // Conversion
        void reload(const std::shared_ptr<std::filesystem::path>&);
        wm::EntitySnapshot convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const;

        // MongoDB connection
        void setTo(const wm::EntitySnapshot&);

    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;
    };
}
