#include "Memory.h"

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::d_ltm
{

    Memory::Memory()
    {
    }

    Memory::Memory(const std::string& name) :
        base::detail::MemoryItem(MemoryID().withMemoryName(name))
    {
    }

    Memory::Memory(const MemoryID& id) :
        base::detail::MemoryItem(id)
    {
    }

    Memory::Memory(const Memory& other) :
        base::detail::MemoryItem(other),
        base::detail::MemoryContainerBase<std::map<std::string, CoreSegment>, Memory>(other)
    {
        *this = other;
    }

    Memory& Memory::operator=(const Memory& other)
    {
        other._copySelf(*this);
        return *this;
    }

    std::filesystem::path Memory::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path Memory::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName;
    }

    wm::Memory Memory::convert() const
    {
        wm::Memory m;
        for (const auto& [_, s] : _container)
        {
            m.addCoreSegment(s.convert());
        }
        return m;
    }

    void Memory::reload(const std::filesystem::path& p)
    {
        if (std::filesystem::is_regular_file(p))
        {
            ARMARX_ERROR << "The entered path is leading to a file! Abort due to error.";
        }

        _container.clear();
        path = std::make_shared<std::filesystem::path>(p.parent_path());

        if (!std::filesystem::exists(p))
        {
            ARMARX_INFO << "The entered path does not exist. Assuming an empty container.";
        }
        else
        {
            id() = MemoryID().withMemoryName(p.filename());

            for (const auto& d : std::filesystem::directory_iterator(p))
            {
                if (d.is_directory())
                {
                    std::string k = d.path().filename();
                    auto wms = _container.emplace(std::make_pair(k, id().withCoreSegmentName(k)));
                    wms.first->second.reload(path);
                }
            }
        }
    }

    void Memory::append(const wm::Memory& m)
    {
        std::filesystem::create_directories(_fullPath());
        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                std::filesystem::create_directory(_fullPath() / k);

                auto wms = _container.emplace(std::make_pair(k, id().withCoreSegmentName(k)));
                wms.first->second.path = path;
                wms.first->second.append(s);
            }
        }
    }
}
