#include "ProviderSegment.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::d_ltm
{

    ProviderSegment::ProviderSegment()
    {
    }

    ProviderSegment::ProviderSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(MemoryID().withProviderSegmentName(name)),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(MemoryID().withProviderSegmentName(name), aronType)
    {
    }

    ProviderSegment::ProviderSegment(const std::string& name, const MemoryID parentID, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(parentID.withProviderSegmentName(name)),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(parentID.withProviderSegmentName(name), aronType)
    {
    }

    ProviderSegment::ProviderSegment(const MemoryID id, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(id),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(id, aronType)
    {
    }

    ProviderSegment::ProviderSegment(const ProviderSegment& other) :
        base::detail::MemoryItem(other),
        base::detail::MemoryContainerBase<std::map<std::string, Entity>, ProviderSegment>(other),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(other)
    {
    }

    ProviderSegment& ProviderSegment::operator=(const ProviderSegment& other)
    {
        other._copySelf(*this);
        return *this;
    }

    std::filesystem::path ProviderSegment::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path ProviderSegment::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName / id().coreSegmentName / id().providerSegmentName;
    }

    wm::ProviderSegment ProviderSegment::convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const
    {
        wm::ProviderSegment m;
        for (const auto& [_, s] : _container)
        {
            if (hasAronType())
            {
                m.addEntity(s.convert(_aronType));
            }
            else
            {
                m.addEntity(s.convert(expectedStructure));
            }
        }
        return m;
    }

    void ProviderSegment::reload(const std::shared_ptr<std::filesystem::path>& p_ptr)
    {
        if (!p_ptr)
        {
            ARMARX_WARNING << "The entered is NULL.";
        }
        std::filesystem::path p = _fullPath(*p_ptr);
        if (std::filesystem::is_regular_file(p))
        {
            ARMARX_ERROR << "The entered path is leading to a file! Abort due to error.";
        }

        _container.clear();
        path = p_ptr;

        if (!std::filesystem::exists(p))
        {
            ARMARX_INFO << "The entered path does not exist. Assuming an empty container.";
        }
        else
        {
            for (const auto& d : std::filesystem::directory_iterator(p))
            {
                if (d.is_directory())
                {
                    std::string k = d.path().filename();
                    auto wms = _container.emplace(std::make_pair(k, id().withEntityName(k)));
                    wms.first->second.reload(p_ptr);
                }

                if (d.is_regular_file())
                {
                    readAronType(d.path());
                }
            }
        }
    }

    void ProviderSegment::append(const wm::ProviderSegment& m)
    {
        std::filesystem::create_directories(_fullPath());
        writeAronType(_fullPath());

        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                std::filesystem::create_directory(_fullPath() / k);
                auto wms = _container.emplace(std::make_pair(k, id().withEntityName(k)));
                wms.first->second.path = path;
                wms.first->second.append(s);
            }
        }
    }
}
