#pragma once

#include <filesystem>

#include "../base/ProviderSegmentBase.h"
#include "detail/TypedEntityContainer.h"

#include "Entity.h"

#include "../workingmemory/ProviderSegment.h"


namespace armarx::armem::d_ltm
{

    /**
     * @brief Data of a provider segment containing multiple entities.
     */
    class ProviderSegment :
        virtual public base::ProviderSegmentBase<Entity, ProviderSegment>,
        virtual public detail::TypedEntityContainer<Entity, ProviderSegment>
    {
        using Base = base::ProviderSegmentBase<Entity, ProviderSegment>;

    public:
        ProviderSegment();
        ProviderSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        ProviderSegment(const std::string& name, const MemoryID parentID, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        ProviderSegment(const MemoryID id, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);

        ProviderSegment(const ProviderSegment& other);
        ProviderSegment& operator=(const ProviderSegment& other);

        // Conversion
        wm::ProviderSegment convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const;

        // Filesystem connection
        void reload(const std::shared_ptr<std::filesystem::path>&);
        void append(const wm::ProviderSegment&);

    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;
    };

}
