#pragma once

#include <iostream>
#include <fstream>

#include "../../base/detail/TypedEntityContainerBase.h"
#include "../Entity.h"

#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/visitor/Visitor.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/converter/Converter.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/reader/nlohmannJSON/NlohmannJSONReader.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/navigator/NavigatorWriter.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/nlohmannJSON/NlohmannJSONWriter.h>

namespace armarx::armem::d_ltm::detail
{

    /**
     * @brief An entity container with a specific (Aron) type.
     */
    template <class _ValueT, class Derived>
    class TypedEntityContainer :
        virtual public base::detail::TypedEntityContainerBase<_ValueT, Entity, Derived>
    {
        using Base = base::detail::TypedEntityContainerBase<_ValueT, Entity, Derived>;

    public:
        TypedEntityContainer& operator=(const TypedEntityContainer& other)
        {
            other._copySelf(*this);
            return *this;
        }

    protected:
        aron::typenavigator::ObjectNavigatorPtr unwrapType(const aron::typenavigator::ObjectNavigatorPtr& t) const
        {
            return aron::typenavigator::ObjectNavigator::DynamicCastAndCheck(t->getMemberType(TYPE_WRAPPER_DATA_FIELD));
        }

        aron::typenavigator::ObjectNavigatorPtr wrapType(const aron::typenavigator::ObjectNavigatorPtr& t) const
        {
            aron::typenavigator::ObjectNavigatorPtr typeWrapped(new aron::typenavigator::ObjectNavigator());
            typeWrapped->setObjectName(t->getObjectName() + "__ltm_type_export");
            typeWrapped->addMemberType(TYPE_WRAPPER_DATA_FIELD, t);

            typeWrapped->addMemberType(TYPE_WRAPPER_TIME_STORED_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
            typeWrapped->addMemberType(TYPE_WRAPPER_TIME_CREATED_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
            typeWrapped->addMemberType(TYPE_WRAPPER_TIME_SENT_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
            typeWrapped->addMemberType(TYPE_WRAPPER_TIME_ARRIVED_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
            typeWrapped->addMemberType(TYPE_WRAPPER_CONFIDENCE_FIELD, std::make_shared<aron::typenavigator::DoubleNavigator>());

            return typeWrapped;
        }

        using Base::aronType;
        void readAronType(const std::filesystem::path& d)
        {
            if (std::filesystem::is_regular_file(d))
            {
                if (d.filename() == (std::string(TYPE_FILENAME) + ".json"))
                {
                    std::ifstream ifs(d);
                    std::string file_content((std::istreambuf_iterator<char>(ifs)), (std::istreambuf_iterator<char>()));

                    aron::typeIO::reader::NlohmannJSONReader typeReader(file_content);
                    aron::typeIO::writer::NavigatorWriter navWriter;
                    aron::typeIO::Converter::ReadAndConvert(typeReader, navWriter);
                    this->aronType() = aron::typenavigator::ObjectNavigator::DynamicCastAndCheck(navWriter.getResult());
                }
            }
        }

        using Base::hasAronType;
        void writeAronType(const std::filesystem::path& d)
        {
            if (hasAronType())
            {
                std::ofstream ofs;
                ofs.open(d);

                aron::typeIO::writer::NlohmannJSONWriter typeWriter;
                aron::typeIO::Visitor::VisitAndSetup(typeWriter, this->aronType());
                std::string new_file_full_content = typeWriter.getResult().dump(2);

                ofs << new_file_full_content;
                ofs.close();
            }
        }

    private:
        static const constexpr char* TYPE_FILENAME = "type";
        static constexpr const char* TYPE_WRAPPER_DATA_FIELD            = "__ARON_DATA";
        static constexpr const char* TYPE_WRAPPER_TIME_STORED_FIELD     = "__WRITER_METADATA__TIME_STORED";
        static constexpr const char* TYPE_WRAPPER_TIME_CREATED_FIELD    = "__ENTITY_METADATA__TIME_CREATED";
        static constexpr const char* TYPE_WRAPPER_TIME_SENT_FIELD       = "__ENTITY_METADATA__TIME_SENT";
        static constexpr const char* TYPE_WRAPPER_TIME_ARRIVED_FIELD    = "__ENTITY_METADATA__TIME_ARRIVED";
        static constexpr const char* TYPE_WRAPPER_CONFIDENCE_FIELD      = "__ENTITY_METADATA__CONFIDENCE";
    };
}
