#include "CoreSegment.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::ltm
{

    CoreSegment::CoreSegment()
    {
    }

    CoreSegment::CoreSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(MemoryID().withCoreSegmentName(name)),
        base::detail::TypedEntityContainerBase<ProviderSegment, Entity, CoreSegment>(MemoryID().withCoreSegmentName(name), aronType)
    {
    }

    CoreSegment::CoreSegment(const std::string& name, const MemoryID& parentID, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(MemoryID().withCoreSegmentName(name)),
        base::detail::TypedEntityContainerBase<ProviderSegment, Entity, CoreSegment>(parentID.withCoreSegmentName(name), aronType)
    {
    }

    CoreSegment::CoreSegment(const MemoryID& id, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(id),
        base::detail::TypedEntityContainerBase<ProviderSegment, Entity, CoreSegment>(id, aronType)
    {
    }

    CoreSegment::CoreSegment(const CoreSegment& other) :
        base::detail::MemoryItem(other),
        base::detail::MemoryContainerBase<std::map<std::string, ProviderSegment>, CoreSegment>(other),
        base::detail::TypedEntityContainerBase<ProviderSegment, Entity, CoreSegment>(other)
    {
    }

    CoreSegment& CoreSegment::operator=(const CoreSegment& other)
    {
        other._copySelf(*this);
        return *this;
    }

    wm::CoreSegment CoreSegment::convert() const
    {
        wm::CoreSegment m;
        for (const auto& [_, s] : _container)
        {
            m.addProviderSegment(s.convert());
        }
        return m;
    }

    void CoreSegment::append(const wm::CoreSegment& m)
    {
        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                auto wms = _container.emplace(std::make_pair(k, id().withCoreSegmentName(k)));
                wms.first->second.append(s);
            }
        }
    }
}
