#include "Memory.h"

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::ltm
{

    Memory::Memory()
    {
    }

    Memory::Memory(const std::string& name) :
        base::detail::MemoryItem(MemoryID().withMemoryName(name))
    {
    }

    Memory::Memory(const MemoryID& id) :
        MemoryItem(id)
    {
    }

    Memory::Memory(const Memory& other) :
        base::detail::MemoryItem(other),
        base::detail::MemoryContainerBase<std::map<std::string, CoreSegment>, Memory>(other)
    {
        *this = other;
    }

    Memory& Memory::operator=(const Memory& other)
    {
        other._copySelf(*this);
        return *this;
    }

    wm::Memory Memory::convert() const
    {
        wm::Memory m;
        for (const auto& [_, s] : _container)
        {
            m.addCoreSegment(s.convert());
        }
        return m;
    }

    void Memory::reload(const MongoDBConnectionManager::MongoDBSettings& settings)
    {
        dbsettings = settings;
        std::cout << "Setting connection to: " << settings.uniqueString() << std::endl;
    }

    void Memory::append(const wm::Memory& m)
    {
        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                auto wms = _container.emplace(std::make_pair(k, id().withCoreSegmentName(k)));
                wms.first->second.append(s);
            }
        }
    }
}
