#include "ProviderSegment.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::ltm
{

    ProviderSegment::ProviderSegment()
    {
    }

    ProviderSegment::ProviderSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(MemoryID().withProviderSegmentName(name)),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(MemoryID().withProviderSegmentName(name), aronType)
    {
    }

    ProviderSegment::ProviderSegment(const std::string& name, const MemoryID parentID, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(parentID.withProviderSegmentName(name)),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(parentID.withProviderSegmentName(name), aronType)
    {
    }

    ProviderSegment::ProviderSegment(const MemoryID id, aron::typenavigator::ObjectNavigatorPtr aronType) :
        base::detail::MemoryItem(id),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(id, aronType)
    {
    }

    ProviderSegment::ProviderSegment(const ProviderSegment& other) :
        base::detail::MemoryItem(other),
        base::detail::MemoryContainerBase<std::map<std::string, Entity>, ProviderSegment>(other),
        base::detail::TypedEntityContainerBase<Entity, Entity, ProviderSegment>(other)
    {
    }

    ProviderSegment& ProviderSegment::operator=(const ProviderSegment& other)
    {
        other._copySelf(*this);
        return *this;
    }

    wm::ProviderSegment ProviderSegment::convert() const
    {
        wm::ProviderSegment m;
        for (const auto& [_, s] : _container)
        {
            m.addEntity(s.convert());
        }
        return m;
    }

    void ProviderSegment::append(const wm::ProviderSegment& m)
    {
        for (const auto& [k, s] : m.container())
        {
            if (const auto& it = _container.find(k); it != _container.end())
            {
                it->second.append(s);
            }
            else
            {
                auto wms = _container.emplace(std::make_pair(k, id().withEntityName(k)));
                wms.first->second.append(s);
            }
        }
    }
}
