#include "EntityInstance.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem::wm
{

    bool EntityInstanceMetadata::operator==(const EntityInstanceMetadata& other) const
    {
        return timeCreated == other.timeCreated
               && timeSent == other.timeSent
               && timeArrived == other.timeArrived
               && std::abs(confidence - other.confidence) < 1e-6f;
    }

    EntityInstance::EntityInstance()
    {
    }

    EntityInstance::EntityInstance(const EntityInstance& other) :
        base::detail::MemoryItem(other.id()),
        _metadata(other._metadata),
        _data(other._data)
    {
    }

    EntityInstance::EntityInstance(int index, const MemoryID& parentID) :
        EntityInstance(parentID.withInstanceIndex(index))
    {
    }

    EntityInstance::EntityInstance(const MemoryID& id) :
        base::detail::MemoryItem(id)
    {
    }

    EntityInstance& EntityInstance::operator=(const EntityInstance& other)
    {
        other._copySelf(*this);
        return *this;
    }


    bool EntityInstance::equalsDeep(const EntityInstance& other) const
    {
        return id() == other.id()
               && _metadata == other.metadata()
               && _data->equalsDeep(other.data());
    }

    void EntityInstance::update(const EntityUpdate& update, int index)
    {
        ARMARX_CHECK_FITS_SIZE(index, update.instancesData.size());

        this->index() = index;
        setData(update.instancesData.at(size_t(index)));

        this->_metadata.confidence = update.confidence;

        this->_metadata.timeCreated = update.timeCreated;
        this->_metadata.timeSent = update.timeSent;
        this->_metadata.timeArrived = update.timeArrived;
    }

    EntityInstance EntityInstance::copy() const
    {
        EntityInstance d;
        this->_copySelf(d);
        return d;
    }

    EntityInstance EntityInstance::copyWithoutData() const
    {
        EntityInstance d;
        this->_copySelfWithoutData(d);
        return d;
    }

    void EntityInstance::_copySelf(EntityInstance& other) const
    {
        EntityInstanceBase<EntityInstance>::_copySelf(other);
        other._metadata = _metadata;
        other._data = _data;
    }

    void EntityInstance::_copySelfWithoutData(EntityInstance& other) const
    {
        EntityInstanceBase<EntityInstance>::_copySelf(other);
    }
}
