#pragma once

#include "../base/EntityInstanceBase.h"

namespace armarx::armem::wm
{

    /**
     * @brief Metadata of an entity instance.
     */
    struct EntityInstanceMetadata
    {
        /// Time when this value was created.
        Time timeCreated;
        /// Time when this value was sent to the memory.
        Time timeSent;
        /// Time when this value has arrived at the memory.
        Time timeArrived;

        /// An optional confidence, may be used for things like decay.
        float confidence = 1.0;


        bool operator==(const EntityInstanceMetadata& other) const;
        inline bool operator!=(const EntityInstanceMetadata& other) const
        {
            return !(*this == other);
        }
    };

    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance :
        virtual public base::EntityInstanceBase<EntityInstance>
    {
        using Base = base::EntityInstanceBase<EntityInstance>;

    public:
        EntityInstance();
        EntityInstance(const EntityInstance&);
        EntityInstance(int index, const MemoryID& parentID = {});
        EntityInstance(const MemoryID& id);

        EntityInstance& operator=(const EntityInstance& other);

        EntityInstanceMetadata& metadata()
        {
            return _metadata;
        }
        inline const EntityInstanceMetadata& metadata() const
        {
            return _metadata;
        }

        inline armarx::aron::datanavigator::DictNavigatorPtr data() const
        {
            return _data;
        }

        void setData(const aron::datanavigator::DictNavigatorPtr& data)
        {
            this->_data = data;
        }


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        virtual void update(const EntityUpdate& update, int index) override;

        virtual bool equalsDeep(const EntityInstance& other) const override;

        virtual EntityInstance copy() const override;
        virtual EntityInstance copyWithoutData() const;

    protected:
        virtual void _copySelf(EntityInstance& other) const override;
        virtual void _copySelfWithoutData(EntityInstance& other) const;


    private:
        /// The metadata.
        EntityInstanceMetadata _metadata;

        /// The data. May be nullptr.
        armarx::aron::datanavigator::DictNavigatorPtr _data;
    };
}
